package com.googlecode.functionalcollections;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.Collection;
import java.util.NoSuchElementException;
import java.util.concurrent.atomic.AtomicInteger;

import org.junit.Test;

import com.google.common.base.Function;
import com.google.common.base.Predicate;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;

public class FunctionalIterablesTest {
	
	@Test
	public void each() {
		final Collection<Object> source = new ArrayList<Object>();
		source.add(new Object());
		source.add(new Object());
		
		final AtomicInteger counter = new AtomicInteger();
		
		FunctionalIterables.make(source).each(new Block<Object>() {
			public void apply(Object input) {
				assertTrue(source.contains(input));
				counter.incrementAndGet();
			}
		});
		
		assertTrue(source.size() == counter.intValue());
	}
	
	@Test
	public void any() {
		final Object obj1 = new Object();
		final Object obj2 = new Object();
		
		Collection<Object> source = new ArrayList<Object>();
		source.add(obj1);
		source.add(obj2);
		
		boolean any = FunctionalIterables.make(source).any(new Predicate<Object>() {
			public boolean apply(Object input) {
				return input == obj1;
			}
		});
		
		assertTrue(any);
	}
	
	@Test
	public void all() {
		final Object obj1 = new Object();
		final Object obj2 = new Object();
		
		Collection<Object> source = new ArrayList<Object>();
		source.add(obj1);
		source.add(obj2);
		
		boolean all = FunctionalIterables.make(source).any(new Predicate<Object>() {
			public boolean apply(Object input) {
				return input == obj1 || input == obj2;
			}
		});
		
		assertTrue(all);
	}
	
	@Test
	public void findSuccess() {
		final Object obj = new Object();
		
		Collection<Object> source = new ArrayList<Object>();
		source.add(obj);
		
		
		Object found = FunctionalIterables.make(source).find(new Predicate<Object>() {
			public boolean apply(Object input) {
				return input == obj;
			}
		});
		
		assertEquals(obj, found);
	}
	
	@Test(expected = NoSuchElementException.class)
	public void findFailed() {
		Collection<Object> source = new ArrayList<Object>();

		FunctionalIterables.make(source).find(new Predicate<Object>() {
			public boolean apply(Object input) {
				return false;
			}
		});
	}
	
	@Test
	public void detectSuccess() {
		final Object obj = new Object();
		
		Collection<Object> source = new ArrayList<Object>();
		source.add(obj);
		
		
		Object found = FunctionalIterables.make(source).detect(new Predicate<Object>() {
			public boolean apply(Object input) {
				return input == obj;
			}
		});
		
		assertEquals(obj, found);
	}
	
	@Test
	public void detectFailed() {
		Collection<Object> source = new ArrayList<Object>();

		Object detected = FunctionalIterables.make(source).detect(new Predicate<Object>() {
			public boolean apply(Object input) {
				return false;
			}
		});
		
		assertNull(detected);
	}
	
	@Test
	public void filter() {
		final Object obj1 = new Object();
		final Object obj2 = new Object();
		final Object obj3 = new Object();
		final Object obj4 = new Object();
		
		Collection<Object> expected = new ArrayList<Object>();
		expected.add(obj1);
		expected.add(obj3);
		
		Collection<Object> source = new ArrayList<Object>();
		source.add(obj1);
		source.add(obj2);
		source.add(obj3);
		source.add(obj4);
		
		FunctionalIterable<Object> filtered = FunctionalIterables.make(source).filter(new Predicate<Object>() {
			public boolean apply(Object input) {
				return input == obj1 || input == obj3;
			}
		});
		
		assertTrue(Iterables.elementsEqual(expected, filtered));
	}
	
	@Test
	public void transform() {
		final Object obj1 = new Object();
		final Object obj2 = new Object();
		final Object obj3 = new Object();
		final Object obj4 = new Object();
		
		Collection<String> expected = new ArrayList<String>();
		expected.add(obj1.toString());
		expected.add(obj2.toString());
		expected.add(obj3.toString());
		expected.add(obj4.toString());
		
		Collection<Object> source = new ArrayList<Object>();
		source.add(obj1);
		source.add(obj2);
		source.add(obj3);
		source.add(obj4);
		
		FunctionalIterable<String> transformed = FunctionalIterables.make(source).transform(new Function<Object, String>() {
			public String apply(Object from) {
				return from.toString();
			}
		});
		
		assertTrue(Iterables.elementsEqual(expected, transformed));
	}
	
	@Test
	public void inject() {
		Collection<Integer> source = new ArrayList<Integer>();
		source.add(1);
		source.add(2);
		source.add(3);
		source.add(4);
		source.add(5);
		source.add(6);
		source.add(7);
		source.add(8);
		source.add(9);
		source.add(10);
		
		Integer initial = 0;
		
		Integer total = FunctionalIterables.make(source).inject(initial, new Injector<Integer, Integer>() {
			public Integer apply(Integer memo, Integer input) {
				return memo + input;
			}
		});
	
		assertTrue(total == 55);
	}
	
	@Test
	public void toCollection() {
		final String obj1 = "obj1";
		final String obj2 = "obj2";
		final String obj3 = "obj3";
		
		Collection<String> source = new ArrayList<String>();
		source.add(obj1);
		source.add(obj2);
		source.add(obj3);
		
		Collection<String> dest = FunctionalIterables.make(source).toCollection();
	
		assertTrue(Iterables.elementsEqual(source, dest));
	}
	
	@Test
	public void uniq() {
		final String obj1 = "obj1";
		
		Collection<String> source = new ArrayList<String>();
		source.add(obj1);
		source.add(obj1);
		source.add(obj1);
		
		FunctionalIterable<String> uniqed = FunctionalIterables.make(source).uniq();
		assertTrue(Iterables.elementsEqual(Lists.newArrayList(obj1), uniqed));
	}
	
	
	@Test
	public void reverse() {
		final String obj1 = "obj1";
		final String obj2 = "obj2";
		final String obj3 = "obj3";
		
		Iterable<String> expected = FunctionalIterables.<String>create().concat(obj3).concat(obj2).concat(obj1);
		
		Collection<String> source = new ArrayList<String>();
		source.add(obj1);
		source.add(obj2);
		source.add(obj3);
		
		FunctionalIterable<String> reversed = FunctionalIterables.make(source).reverse();
		
		assertTrue(Iterables.elementsEqual(expected, reversed));
	}
	
	@Test
	public void messagedIterator() {
		String message = "No available element.";
		try {
			FunctionalIterables.create().messagedIterator(message).next();
		} catch (NoSuchElementException e) {
			assertEquals(message, e.getMessage());
		}
	}
	
	
	@Test
	public void concat() {
		FunctionalIterable<Integer> expected = FunctionalIterables.make(1, 2, 3, 4, 5);
		FunctionalIterable<Integer> source = FunctionalIterables.make(1, 2, 3);
		source.concat(4).concat(5);
		assertTrue(Iterables.elementsEqual(expected, source));
	}
	
	
//	@Test
//	public void flatten() {
//		Iterable<Integer> expected = Lists.newArrayList(1, 2, 3, 4, 5, 6, 7, 8, 9);
//		
//		Iterable<Object> source = new ArrayList<Object>();
//		
//		FunctionalIterable<Object> made = FunctionalIterables.make(source);
//		
//		Iterable<Object> flattened = made.flatten();
//		
//		assertTrue(Iterables.elementsEqual(expected, flattened));
//	}
	
}
